# TODO
# - unpackaged files
#   /usr/share/doc/node/gdbinit
#   /usr/share/systemtap/tapset/node.stp

# Conditional build:
%bcond_without	system_uv	# system uv
%bcond_with	shared	# build libnode.so shared library

# NOTES:
# - https://nodejs.org/en/download/releases/

# NODE_MODULE_VERSION refers to the ABI (application binary interface) version
# number of Node.js, used to determine which versions of Node.js compiled C++
# add-on binaries can be loaded in to without needing to be re-compiled. It
# used to be stored as hex value in earlier versions, but is now represented as
# an integer.
%define		node_module_version	48
Summary:	Asynchronous JavaScript Engine
Name:		nodejs
# 6.9 is LTS, Active LTS: 2018-04-18, EOL: 2019-04-18
# https://github.com/nodejs/LTS
Version:	6.9.4
Release:	1
License:	BSD and MIT and Apache v2.0 and GPL v3
Group:		Development/Languages
Source0:	https://nodejs.org/dist/v%{version}/node-v%{version}.tar.gz
# Source0-md5:	3795199b5950b25179248847b1a5fc86
Patch1:		%{name}-shared.patch
# force node to use /usr/lib/node as the systemwide module directory
Patch2:		%{name}-libpath.patch
# use /usr/lib64/node as an arch-specific module dir when appropriate
Patch3:		%{name}-lib64path.patch
Patch4:		%{name}-use-system-certs.patch
Patch5:		uv-fpic.patch
URL:		https://nodejs.org/
BuildRequires:	gcc >= 5:4.0
BuildRequires:	http-parser-devel >= 2.7.0
BuildRequires:	libstdc++-devel
%{?with_system_uv:BuildRequires:	libuv-devel >= 1.6.0}
BuildRequires:	openssl-devel >= 1.0.1
BuildRequires:	pkgconfig
BuildRequires:	python >= 1:2.7
BuildRequires:	python-jsmin
BuildRequires:	python-modules
BuildRequires:	rpm >= 4.4.9-56
BuildRequires:	rpmbuild(macros) >= 1.219
BuildRequires:	sed >= 4.0
BuildRequires:	zlib-devel
Requires:	ca-certificates
Provides:	nodejs(engine) = %{version}
Provides:	nodejs(module-version) = %{node_module_version}
Obsoletes:	nodejs-waf
ExclusiveArch:	%{ix86} %{x8664} arm
BuildRoot:	%{tmpdir}/%{name}-%{version}-root-%(id -u -n)

%define		sover	%(echo %{version} | cut -d. -f2)
# add macro, so adapter won't replace it back literal
%define		doc_ver	%{version}

%description
Node.js is a platform built on Chrome's JavaScript runtime for easily
building fast, scalable network applications. Node.js uses an
event-driven, non-blocking I/O model that makes it lightweight and
efficient, perfect for data-intensive real-time applications that run
across distributed devices.

%package devel
Summary:	Development headers for nodejs
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
Requires:	gcc
Requires:	http-parser-devel >= 2.5.0
Requires:	libstdc++-devel
%{?with_system_uv:Requires:	libuv-devel >= 1.6.0}
Requires:	openssl-devel
Requires:	zlib-devel

%description devel
Development headers for nodejs.

%package doc
Summary:	Evented I/O for V8 JavaScript - documentation
Group:		Documentation
URL:		https://nodejs.org/dist/v%{doc_ver}/docs/api
%if "%{_rpmversion}" >= "5"
BuildArch:	noarch
%endif

%description doc
Node.js is a server-side JavaScript environment that uses an
asynchronous event-driven model. Node's goal is to provide an easy way
to build scalable network programs.

This package contains the documentation for nodejs.

%prep
%setup -q -n node-v%{version}
%{?with_shared:%patch1 -p1}
#%patch1 -p1
%if %{_lib} == "lib64"
%patch3 -p1
%else
%patch2 -p1
%endif
%patch4 -p1
rm src/node_root_certs.h
#%{?with_system_uv:%patch5 -p1}

grep -r '#!.*env python' -l . | xargs %{__sed} -i -e '1 s,#!.*env python,#!%{__python},'

rm -r deps/npm
rm -r deps/http_parser
rm -r deps/openssl
%{?with_system_uv:rm -r deps/uv}
rm -r deps/zlib

%build
ver=$(awk '/#define NODE_MODULE_VERSION/{print $3}' src/node_version.h)
test "$ver" = "%{node_module_version}"

# CC used only to detect if CC is clang, not used for compiling
CC="%{__cc}" \
CXX="%{__cxx}" \
GYP_DEFINES="soname_version=%{sover}" \
./configure \
	--shared-zlib \
	--shared-openssl \
	%{?0:--shared-cares} \
	%{?with_system_uv:--shared-libuv} \
	--shared-http-parser \
	--without-npm \
	--without-dtrace \
	--prefix=%{_prefix}

# add LFS defines from libuv (RHBZ#892601)
# CXXFLAGS must be exported, as it is needed for make, not gyp
CXXFLAGS="%{rpmcxxflags} -D_LARGEFILE_SOURCE -D_FILE_OFFSET_BITS=64 -fPIC" \
LDFLAGS="%{rpmldflags}" \
%{__make} -C out V=1 BUILDTYPE=Release

%install
rm -rf $RPM_BUILD_ROOT
%{__python} tools/install.py install "$RPM_BUILD_ROOT" "%{_prefix}"

%if %{with shared}
lib=$(basename $RPM_BUILD_ROOT%{_libdir}/libnode.so.*.*.*)
ln -s $lib $RPM_BUILD_ROOT%{_libdir}/libnode.so.10
ln -s $lib $RPM_BUILD_ROOT%{_libdir}/libnode.so
%endif

echo '.so man1/node.1' > $RPM_BUILD_ROOT%{_mandir}/man1/nodejs.1

install -d $RPM_BUILD_ROOT%{_includedir}/node
cp -p src/*.h $RPM_BUILD_ROOT%{_includedir}/node

# install for node-gyp
install -d $RPM_BUILD_ROOT%{_usrsrc}/%{name}
cp -p common.gypi $RPM_BUILD_ROOT%{_usrsrc}/%{name}
ln -s %{_includedir}/node $RPM_BUILD_ROOT%{_usrsrc}/%{name}/src

# for compat of fedora derivered scripts (shebangs)
ln -s node $RPM_BUILD_ROOT%{_bindir}/nodejs

# globally installed node modules (noarch)
install -d $RPM_BUILD_ROOT%{_prefix}/lib/node_modules

# default searchpaths
install -d $RPM_BUILD_ROOT{%{_libdir},%{_prefix}/lib}/node

# create pkgconfig
install -d $RPM_BUILD_ROOT%{_pkgconfigdir}
cat <<'EOF' > $RPM_BUILD_ROOT%{_pkgconfigdir}/%{name}.pc
version=%{version}
prefix=%{_prefix}
libdir=${prefix}/%{_lib}
includedir=${prefix}/include/node

Name: nodejs
Description: Evented I/O for V8 JavaScript.
Version: ${version}
Cflags: -I${includedir}
EOF

# install documentation
install -d $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}
cp -a doc/api/* $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}
rm $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}/*.md
rm $RPM_BUILD_ROOT%{_docdir}/%{name}-doc-%{version}/*.json

%clean
rm -rf $RPM_BUILD_ROOT

%if %{with shared}
%post	-p /sbin/ldconfig
%postun -p /sbin/ldconfig
%endif

%files
%defattr(644,root,root,755)
%doc README.md AUTHORS CHANGELOG.md ROADMAP.md LICENSE
%attr(755,root,root) %{_bindir}/node
%attr(755,root,root) %{_bindir}/nodejs
%if %{with shared}
%attr(755,root,root) %{_libdir}/libnode.so.*.*.*
%ghost %{_libdir}/libnode.so.10
%endif
%if "%{_lib}" != "lib"
%dir %{_libdir}/node
%endif
%dir %{_prefix}/lib/node
%dir %{_prefix}/lib/node_modules
%{_mandir}/man1/node.1*
%{_mandir}/man1/nodejs.1

%files devel
%defattr(644,root,root,755)
%if %{with shared}
%{_libdir}/libnode.so
%endif
%{_includedir}/node
%{_pkgconfigdir}/nodejs.pc
%{_usrsrc}/%{name}

%files doc
%defattr(644,root,root,755)
%doc %{_docdir}/%{name}-doc-%{version}
%changelog
* Thu Jan 12 2017 PLD Linux Team <feedback@pld-linux.org>
- For complete changelog see: http://git.pld-linux.org/?p=packages/nodejs.git;a=log;h=master

* Thu Jan 12 2017 Elan Ruusamäe <glen@delfi.ee> db609ce
- up to 6.9.4

* Fri Nov 25 2016 Elan Ruusamäe <glen@delfi.ee> b65ad44
- up to 6.9.1-LTS: Active LTS: 2018-04-18, EOL: 2019-04-18

* Tue Oct 18 2016 Elan Ruusamäe <glen@delfi.ee> e9b6043
- a bit doc for NODE_MODULE_VERSION

* Sat Sep 10 2016 Elan Ruusamäe <glen@delfi.ee> 91adacc
- restore python epoch

* Fri Sep 09 2016 mis <mistoo@gmail.com> 318ea6d
- up to 4.5.0 (LTS)

* Fri Apr 15 2016 Elan Ruusamäe <glen@delfi.ee> c8ef655
- update node_module_version to 46

* Wed Mar 30 2016 Elan Ruusamäe <glen@delfi.ee> 145d5ed
- add 4.2.6 to th

* Tue Mar 29 2016 Elan Ruusamäe <glen@delfi.ee> 8a00575
- up to v4.2.6 'Argon' (LTS), 2016-01-21

* Sat Mar 05 2016 Elan Ruusamäe <glen@delfi.ee> 6fd9aec
- openssl 1.0.2g rebuild
- release 2 (by relup.sh)

* Thu Jan 07 2016 Elan Ruusamäe <glen@delfi.ee> e606902
- up to 4.2.4

* Thu Jan 07 2016 Elan Ruusamäe <glen@delfi.ee> 05e85ef
- up to 0.10.41, add link in doc package

* Fri Jan 01 2016 Elan Ruusamäe <glen@delfi.ee> a582f31
- update BR

* Fri Jan 01 2016 Elan Ruusamäe <glen@delfi.ee> 1f0413d
- ensure bundled ssl certs are not present

* Fri Jan 01 2016 Elan Ruusamäe <glen@delfi.ee> 94f7bd5
- no system c-ares support

* Fri Jan 01 2016 Elan Ruusamäe <glen@delfi.ee> 87fb6ec
- drop outdated patches

* Fri Jan 01 2016 Elan Ruusamäe <glen@delfi.ee> 9868c8d
- drop system v8 support; upstream has dropped any support

* Fri Jan 01 2016 Jan Rękorajski <baggins@pld-linux.org> 29fa0dd
- release 4 (by relup.sh)

* Fri Jan 01 2016 Jan Rękorajski <baggins@pld-linux.org> 1ae0074
- disable system uv (does not build with current libuv)

* Fri Dec 25 2015 Jan Rękorajski <baggins@pld-linux.org> 53a9ac7
- release 3 (by relup.sh)

* Thu Dec 24 2015 Jan Rękorajski <baggins@pld-linux.org> 52e8466
- release 2 (by relup.sh)

